# Copyright (c) 2023 SUSE LLC.  All rights reserved.
#
# This file is part of mash.
#
# mash is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# mash is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with mash.  If not, see <http://www.gnu.org/licenses/>
#

from mash.services.api.v1.schema import (
    email,
    non_empty_string,
    string_with_example
)

image_condition = {
    'type': 'object',
    'properties': {
        'package_name': string_with_example(
            'kernel-default',
            description='The name of the package for this condition. '
                        'If no name is provided for a condition then the '
                        'condition is checked against the image itself.'
        ),
        'version': string_with_example(
            '4.13.1',
            description='The package or image version from the build service.'
                        ' If no package_name is provided with the condition '
                        'then the condition is against the image.'
        ),
        'release': string_with_example(
            '1.1',
            description='The build (release) number for the package or image.'
                        ' If no package_name is provided with the condition '
                        'then the condition is against the image.'
        ),
        'condition': {
            'type': 'string',
            'enum': ['>=', '==', '<=', '>', '<'],
            'example': '==',
            'description': 'The expression to use for comparing version '
                           'and/or release.'
        }
    },
    'additionalProperties': False,
    'anyOf': [
        {'required': ['package_name']},
        {'required': ['version']},
        {'required': ['release']}
    ]
}

utctime = {
    'type': 'string',
    'description': 'An RFC3339 date-time string or "now".'
                   'If using a date string it must be in the future '
                   'and the job will start no sooner than the provided '
                   'date. Now jobs will run as soon as possible.',
    'format': 'regex',
    'pattern': r'^([0-9]+)-(0[1-9]|1[012])-(0[1-9]|[12][0-9]'
               r'|3[01])[Tt]([01][0-9]|2[0-3]):([0-5][0-9]):'
               r'([0-5][0-9]|60)(\.[0-9]+)?(([Zz])|([\+|\-]'
               r'([01][0-9]|2[0-3]):[0-5][0-9]))$|^(now)$',
    'example': '2019-04-28T06:44:50.142Z',
    'examples': ['now', '2019-04-28T06:44:50.142Z']
}
base_job_message = {
    'type': 'object',
    'properties': {
        'last_service': {
            'type': 'string',
            'example': 'create',
            'description': 'The last service in the pipeline to be executed. '
                           'All services except the download service are valid '
                           'values.'
        },
        'utctime': utctime,
        'image': string_with_example(
            'openSUSE-Leap-15.0-EC2-HVM',
            description='This should match the name of the tarball file on'
                        'the download server prior to the architecture. For '
                        'a file like openSUSE-Leap-15.0-EC2-HVM.x86_64-1.0.0-'
                        'Build1.206.vhdfixed.xz.sha256 The "image" is '
                        'openSUSE-Leap-15.0-EC2-HVM.'
        ),
        'cloud_image_name': string_with_example(
            'openSUSE-Leap-15.0-v{date}-hvm-ssd-x86_64',
            description='The name to use for the uploaded image in the cloud '
                        'framework. The name can have a date of upload '
                        'inserted such as the example. The {date} will be '
                        'replaced by the current iso date format at time of '
                        'upload (20200410).'
        ),
        'old_cloud_image_name': string_with_example(
            'openSUSE-Leap-15.0-v20190313-hvm-ssd-x86_64',
            description='The cloud image name for the image to be deprecated. '
                        'This is only required for jobs that run through '
                        'deprecate service.'
        ),
        'conditions': {
            'type': 'array',
            'items': image_condition,
            'minItems': 1,
            'example': [
                {
                    'package_name': 'kernel-default',
                    'version': '4.13.1',
                    'condition': '>='
                }
            ],
            'description': 'A list of image conditions to check the image '
                           'against. Providing only the package name will '
                           'ensure the package is in the image. The version '
                           'is the package version from the build service '
                           'whereas the release is the build or release '
                           'number. At least one of package_name, version or '
                           'release is required. If no package name is '
                           'provided then the condition is against the image '
                           'itself. Valid condition operators are >, <, >=, '
                           '<=, or ==.'
        },
        'download_account': string_with_example(
            'my_aws_account',
            description='The cloud framework account as configured with '
                        'the mash account add client command when the mash '
                        'user was setup. The credentials associated with '
                        'this cloud framework account will be used for the '
                        'image download.'
        ),
        'download_type': string_with_example(
            'OBS',
            description='Service from which to download the image file.'
                        'Supported values are:'
                        '- `OBS`: (default) the image is downloaded from an'
                        ' OpenBuildService instance.'
                        '- `S3` the image is downloaded from an S3 API'
                        ' compatible storage service. '
        ),
        'download_url': string_with_example(
            'https://download.opensuse.org/repositories/'
            'Cloud:/Images:/Leap_15.0/images/',
            description='The URL to a download repository.'
                        'In the case of OBS download, the URL is '
                        'expected to have the image tarball, checksum and '
                        'a packages file.'
                        'For the S3 download case, it contains the URL of the'
                        'S3 bucket containing the image file.'
        ),
        'image_description': string_with_example(
            'openSUSE Leap 15.0 (HVM, 64-bit, SSD-Backed)',
            description='Description to use when creating the image in the '
                        'cloud framework.'
        ),
        'distro': {
            'type': 'string',
            'enum': ['opensuse_leap', 'sles', 'sle_micro', 'rhel'],
            'example': 'sles',
            'description': 'The distribution setting used for the img-proof '
                           'tests.'
        },
        'instance_type': string_with_example(
            't2.micro',
            description='Instance size/type img-proof will use when '
                        'launching a test instance. If no type is provided '
                        'a random type will be selected from a pre-configured '
                        'list.'
        ),
        'ssh_user': string_with_example(
            'root',
            description='The SSH user name that will be used when connecting '
                        'to a test instance.'
        ),
        'tests': {
            'type': 'array',
            'items': non_empty_string,
            'minItems': 1,
            'example': ['test_sles'],
            'description': 'This is a list of img-proof tests or test '
                           'descriptions. The tests will be run against '
                           'the instance. If any tests fail the job will '
                           'fail.'
        },
        'cleanup_images': {
            'type': 'boolean',
            'example': True,
            'description': 'Whether to cleanup the image artifacts. By '
                           'default all artifacts are cleaned up when the '
                           'last service is test or if a publishing job '
                           'fails.'
        },
        'cloud_architecture': {
            'type': 'string',
            'enum': ['x86_64', 'aarch64'],
            'example': 'x86_64',
            'description': 'The underlying architecture for the image. '
                           'Valid options are x86_64 and aarch64 and the '
                           'default architecture is x86_64.'
        },
        'notification_email': email,
        'notify': {
            'type': 'boolean',
            'example': True,
            'description': 'If True Send email notification when job finishes.'
        },
        'profile': string_with_example('Proxy'),
        'conditions_wait_time': {
            'type': 'integer',
            'minimum': 0,
            'example': 900,
            'description': 'The time (in seconds) to wait before failing '
                           'a job on image conditions.'
        },
        'raw_image_upload_type': string_with_example(
            's3bucket',
            description='Cloud framework to use for raw image tarball '
                        'upload. The image tarball will be uploaded as is '
                        'to the framework using the cloud image name.'
        ),
        'raw_image_upload_location': string_with_example(
            'my-bucket/prefix/',
            description='The location to put the raw image. This is '
                        'dependent on the cloud framework chosen. For '
                        'example an S3 upload format would be '
                        '{bucket_name}/{prefix}/ to denote the bucket and '
                        'a prefix where the image should be uploaded.'
        ),
        'raw_image_upload_account': string_with_example(
            'my_aws_account',
            description='The cloud framework account as configured with '
                        'the mash account add client command when the mash '
                        'user was setup. The credentials associated with '
                        'this cloud framework account will be used for the '
                        'raw image upload.'
        ),
        'disallow_licenses': {
            'type': 'array',
            'items': non_empty_string,
            'minItems': 1,
            'example': ['MIT'],
            'description': 'A list of licenses that should not be in the '
                           'image. If a package is in the manifest that has '
                           'a matching license the job will fail.'
        },
        'disallow_packages': {
            'type': 'array',
            'items': non_empty_string,
            'minItems': 1,
            'example': ['*-mini'],
            'description': 'A list of packages that should not be in the '
                           'image. If a package is in the manifest that has '
                           'a matching name the job will fail. The * wildcard '
                           'can be used to match a package name pattern.'
        },
        'boot_firmware': {
            'type': 'array',
            'items': {
                'type': 'string',
                'enum': ['bios', 'uefi', 'uefi-preferred'],
            },
            'example': ['bios'],
            'description': 'A list of boot firmware settings to test the '
                           'image with. The default value if left empty is '
                           'bios only. If both bios and uefi are provided '
                           'the image is tested once for each firmware '
                           'setting.'
        },
        'additional_uploads': {
            'type': 'array',
            'items': {
                'type': 'string',
                'enum': ['sha256', 'sha256.asc']
            },
            'example': ['sha256.asc'],
            'description': 'A list of additional image artifacts to upload '
                           'to the cloud provider bucket alongside the '
                           'image tarball. This allows the checksum and/or '
                           'signature to also be uploaded for image '
                           'verification.'
        },
        'dry_run': {
            'type': 'boolean',
            'example': True,
            'description': 'Only validate the job document and return. '
                           'Do not create the job.'
        },
        'use_build_time': {
            'type': 'boolean',
            'example': True,
            'description': 'If True and the image build time is available '
                           'it will be used when formatting the cloud image '
                           'name. A {date} format is required in the cloud '
                           'image name for the timestamp to be used. If True '
                           'and build time is not available the job will '
                           'fail.'
        },
        'force_replace_image': {
            'type': 'boolean',
            'example': True,
            'description': 'If an image already exists with the same name '
                           'delete the image and tarball from storage. '
                           'Default is False and the job will fail if the '
                           'image already exists.'
        }
    },
    'additionalProperties': False,
    'required': [
        'last_service',
        'utctime',
        'image',
        'download_url'
    ]
}
